----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    23:57:51 06/10/2012 
-- Design Name:    m25p32 wishbone access
-- Module Name:    m25p32_core - Behavioral 
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions: 
-- Description:    This is the logic that governs the access to the m25p32 Non
--                 volatile Flash memory
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use work.m25p32_pkg.ALL;

entity m25p32_core is
   port (
           wb_rst_i : in  STD_LOGIC;
           wb_clk   : in  STD_LOGIC;

           mosi_o : out STD_LOGIC; 
           miso_i : in  STD_LOGIC;  
           sclk_o : out STD_LOGIC;  
           ss_n_o : out STD_LOGIC; 

           CTR0_o : out STD_LOGIC_VECTOR(7  downto 0);
           FMOH_i : in  STD_LOGIC_VECTOR(31 downto 0)
        );
end m25p32_core;

architecture Behavioral of m25p32_core is

   component spi_master_core is
      generic(
      g_inst_width := 1;
      g_addr_width := 3;
      g_data_width := 256
   ); port(
             rst_i    : in  STD_LOGIC;
             clk      : in  STD_LOGIC

             inst_i        : in  STD_LOGIC_VECTOR (8*g_inst_width - 1 downto 0);
             addr_i        : in  STD_LOGIC_VECTOR (8*g_addr_width - 1 downto 0);
             data_i        : in  STD_LOGIC_VECTOR (8*g_data_width - 1 downto 0);
             push_inst_i   : in  STD_LOGIC;
             push_addr_i   : in  STD_LOGIC;
             push_data_i   : in  STD_LOGIC;

             data_i        : in  STD_LOGIC_VECTOR (8*g_data_width - 1 downto 0);
             SPI0_i        : in  STD_LOGIC_VECTOR (31 downto 0)
          );
   end component;


type t_m25p32_fsm  is (S0_IDLE, S1_WREN, S2_SPI_WRITE, S3_DEL_PROG_CYCLE, S4_WRDI, Q1_ERROR);

signal MEM_fsm : t_m25p32_fsm;

signal s_CTR0 : r_CTR0;
signal s_FMOH : r_FMOH;


begin

   s_FMOH    <= f_FMOH(FMOH_i);
   CTR0_o    <= f_STD_LOGIC_VECTOR(s_CTR0);

   p_out: process (wb_clk)
   begin
      if rising_edge(wb_clk) then
         if wb_rst_i = '1' then
            s_CTR0   <= c_CTR0_default;
         else
            if s_FMOH.OPP = '1' then
               case s_FMOH.OP is
                  when ERS    =>
                     case MEM_fsm is
                        when S0_IDLE        =>
                           s_CTR0.OPA     <= '1';
                           s_CTR0.OPF     <= '0';
                        when S1_WREN           =>
                        when S2_SPI_WRITE      =>
                        when S3_DEL_PROG_CYCLE =>
                        when S4_WRDI        =>
                        when Q1_ERROR       =>
                           s_CTR0.OPA     <= '1';
                           s_CTR0.OPF     <= '1';
                        when others            =>
                     end case;
                  when RDP    =>
                     case MEM_fsm is
                        when S0_IDLE           =>
                        when S1_WREN           =>
                        when S2_SPI_WRITE      =>
                        when S3_DEL_PROG_CYCLE =>
                        when S4_WRDI           =>
                        when Q1_ERROR          =>
                        when others            =>
                     end case;

                  when WRP    =>
                     case MEM_fsm is
                        when S0_IDLE           =>
                        when S1_WREN           =>
                        when S2_SPI_WRITE      =>
                        when S3_DEL_PROG_CYCLE =>
                        when S4_WRDI           =>
                        when Q1_ERROR          =>
                        when others            =>
                     end case;

                  when others =>
               end case;
            else
               s_CTR0.OPA     <= '0';
               s_CTR0.OPF     <= '0';
            end if;
         end if;
      else
      end if;
   end process;

   p_ERS_fsm: process (wb_clk)
   begin
      if rising_edge(wb_clk) then
         if wb_rst_i = '1' then
            MEM_fsm  <= S0_IDLE;
         else
            if s_FMOH.OPP = '1' and s_CTR0.OPA = '0' then
               case s_FMOH.OP is
                  when ERS    =>
                     case MEM_fsm is
                        when S0_IDLE           =>
                        when S1_WREN           =>
                        when S2_SPI_WRITE      =>
                        when S3_DEL_PROG_CYCLE =>
                        when S4_WRDI           =>
                        when Q1_ERROR          =>
                        when others            =>
                     end case;
                  when RDP    =>

                  when WRP    =>

                  when others =>
               end case;
            else
            end if;
         end if;
      else
      end if;
   end process;

end Behavioral;
