----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    23:55:10 06/10/2012 
-- Design Name:    m25p32 wishbone access
-- Module Name:    m25p32_regs - Behavioral 
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions: 
-- Description:    This is a wishbone slave able to access m25p32 Non volatile
--                 Flash devices and manage the write and read to the memory.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use work.m25p32_pkg.ALL;

entity m25p32_regs is
   port (
           wb_rst_i   : in  STD_LOGIC; 
           wb_clk     : in  STD_LOGIC; 

           wb_we_i    : in  STD_LOGIC;  
           wb_stb_i   : in  STD_LOGIC;
           wb_cyc_i   : in  STD_LOGIC;
           wb_sel_i   : in  STD_LOGIC_VECTOR (3  downto 0);
           wb_data_i  : in  STD_LOGIC_VECTOR (31 downto 0); 
           wb_data_o  : out STD_LOGIC_VECTOR (31 downto 0); 
           wb_addr_i  : in  STD_LOGIC_VECTOR (3  downto 0); 
           wb_ack_o   : out STD_LOGIC;
           wb_rty_o   : out STD_LOGIC;
           wb_err_o   : out STD_LOGIC;

           CTR0_i     : in   STD_LOGIC_VECTOR (7  downto 0);
           FMOH_o     : out  STD_LOGIC_VECTOR (31 downto 0)
        );
end m25p32_regs;

architecture Behavioral of m25p32_regs is

   signal s_CTR0        : r_CTR0;
   signal s_FMOH        : r_FMOH;

   signal s_wb_ack_o    : STD_LOGIC;
   signal s_wb_rty_o    : STD_LOGIC;
   signal s_wb_err_o    : STD_LOGIC;

begin

   wb_ack_o <= s_wb_ack_o;
   wb_rty_o <= s_wb_rty_o;
   wb_err_o <= s_wb_err_o;

   p_wbslave:   process (wb_clk)
   begin
      if rising_edge(wb_clk) then
         if wb_rst_i = '1' then
            s_CTR0       <=  c_CTR0_default; 
            s_FMOH       <=  c_FMOH_default;
            s_wb_ack_o   <= '0'; 
            s_wb_rty_o   <= '0'; 
            s_wb_err_o   <= '0'; 
         else
            if s_CTR0.OPF = '1' then
               s_FMOH.OPP <= '0';
            else
            end if;

            if (wb_stb_i = '1' and wb_cyc_i = '1') then
               if (s_wb_ack_o or s_wb_rty_o or s_wb_err_o) = '1' then
                  s_wb_ack_o  <= '0'; 
                  s_wb_rty_o  <= '0'; 
                  s_wb_err_o  <= '0'; 
               else
                  case wb_we_i is
                     when '1'    =>
                        case wb_addr_i(3 downto 0) is
                           when c_FMOH_addr   =>
                              if s_FMOH.OPP = '0' then
                                 s_FMOH      <= f_FMOH(wb_data_i);
                                 s_wb_ack_o  <= '1';
                                 s_wb_rty_o  <= '0';
                                 s_wb_err_o  <= '0';
                              else
                                 s_wb_ack_o  <= '0';
                                 s_wb_rty_o  <= '0';
                                 s_wb_err_o  <= '1';
                              end if;
                           when others      =>
                              s_wb_ack_o  <= '0';
                              s_wb_rty_o  <= '0';
                              s_wb_err_o  <= '1';
                        end case;
                     when others =>
                        case wb_addr_i is
                           when c_CTR0_addr   =>
                              wb_data_o(7  downto 0)   <= f_STD_LOGIC_VECTOR(s_CTR0);
                              wb_data_o(31 downto 8)   <= (others => '0');
                              s_wb_ack_o  <= '1';
                              s_wb_rty_o  <= '0';
                              s_wb_err_o  <= '0';
                           when c_FMOH_addr   =>
                              wb_data_o   <= f_STD_LOGIC_VECTOR(s_FMOH);
                              s_wb_ack_o  <= '1';
                              s_wb_rty_o  <= '0';
                              s_wb_err_o  <= '0';
                           when others      =>
                              s_wb_ack_o  <= '0';
                              s_wb_rty_o  <= '0';
                              s_wb_err_o  <= '1';
                        end case;
                  end case;
               end if;
            else
            end if;
         end if;
      else
      end if;
   end process;

end Behavioral;
