----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    01:22:27 06/11/2012 
-- Design Name:    m25p32 top level
-- Module Name:    m25p32_top - Behavioral 
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use work.m25p32_pkg.ALL;

entity m25p32_top is
   port ( 
           wb_rst_i       : in  STD_LOGIC;
           wb_clk         : in  STD_LOGIC;

           wb_we_i        : in  STD_LOGIC;
           wb_stb_i       : in  STD_LOGIC;
           wb_cyc_i       : in  STD_LOGIC;
           wb_sel_i       : in STD_LOGIC_VECTOR (3  downto 0);
           wb_data_i      : in  STD_LOGIC_VECTOR(31 downto 0); 
           wb_data_o      : out STD_LOGIC_VECTOR(31 downto 0); 
           wb_addr_i      : in  STD_LOGIC_VECTOR(3  downto 0); 
           wb_ack_o       : out STD_LOGIC; 
           wb_rty_o       : out STD_LOGIC; 
           wb_err_o       : out STD_LOGIC;

           prom_mosi_o    : out STD_LOGIC;
           prom_cclk_o    : out STD_LOGIC;
           prom_cs0_b_n_o : out STD_LOGIC;
           prom_din_i     : in  STD_LOGIC
        );
end m25p32_top;

architecture Behavioral of m25p32_top is

   signal s_CTR0 : STD_LOGIC_VECTOR(7  downto 0);
   signal s_FMOH : STD_LOGIC_VECTOR(31 downto 0);

   component m25p32_core is
      port (
              wb_rst_i : in  STD_LOGIC;
              wb_clk   : in  STD_LOGIC;

              mosi_o   : out STD_LOGIC;
              miso_i   : in  STD_LOGIC;
              sclk_o   : out STD_LOGIC;
              ss_n_o   : out STD_LOGIC;

              CTR0_o   : out STD_LOGIC_VECTOR (7  downto 0);
              FMOH_i   : in  STD_LOGIC_VECTOR (31 downto 0)
           );
   end component;

   component m25p32_regs is
      port (
              wb_rst_i   : in  STD_LOGIC;
              wb_clk     : in  STD_LOGIC;

              wb_we_i    : in  STD_LOGIC;
              wb_stb_i   : in  STD_LOGIC;
              wb_cyc_i   : in  STD_LOGIC;
              wb_sel_i   : in  STD_LOGIC_VECTOR (3  downto 0);
              wb_data_i  : in  STD_LOGIC_VECTOR (31 downto 0);
              wb_data_o  : out STD_LOGIC_VECTOR (31 downto 0);
              wb_addr_i  : in  STD_LOGIC_VECTOR (3  downto 0);
              wb_ack_o   : out STD_LOGIC;
              wb_rty_o   : out STD_LOGIC;
              wb_err_o   : out STD_LOGIC;

              CTR0_i     : in   STD_LOGIC_VECTOR (7  downto 0);
              FMOH_o     : out  STD_LOGIC_VECTOR (31 downto 0)
           );
   end component;

begin

   inst_m25p32_core: m25p32_core
   port map (
               wb_rst_i   => wb_rst_i, 
               wb_clk     => wb_clk, 

               mosi_o     => prom_mosi_o, 
               miso_i     => prom_din_i, 
               sclk_o     => prom_cclk_o, 
               ss_n_o     => prom_cs0_b_n_o, 

               CTR0_o     => s_CTR0, 
               FMOH_i     => s_FMOH
            );

   inst_m25p32_regs:   m25p32_regs
   port map (
               wb_rst_i   => wb_rst_i, 
               wb_clk     => wb_clk, 

               wb_we_i    => wb_we_i, 
               wb_stb_i   => wb_stb_i, 
               wb_cyc_i   => wb_cyc_i, 
               wb_sel_i   => wb_sel_i, 
               wb_data_i  => wb_data_i,
               wb_data_o  => wb_data_o,
               wb_addr_i  => wb_addr_i,
               wb_ack_o   => wb_ack_o,
               wb_rty_o   => wb_rty_o,
               wb_err_o   => wb_err_o,

               CTR0_i     => s_CTR0,
               FMOH_o     => s_FMOH
            );

end Behavioral;
