----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:34:08 06/14/2012 
-- Design Name: 
-- Module Name:    spi_master_regs - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use work.spi_master_pkg.ALL;


entity spi_master_regs is
   port(
              wb_rst_i   : in  STD_LOGIC; 
              wb_clk_i   : in  STD_LOGIC; 

              wb_stb_i   : in  STD_LOGIC; 
              wb_cyc_i   : in  STD_LOGIC; 
              wb_sel_i   : in  STD_LOGIC_VECTOR (3  downto 0); 
              wb_we_i    : in  STD_LOGIC;
              wb_data_i  : in  STD_LOGIC_VECTOR (31 downto 0); 
              wb_data_o  : out STD_LOGIC_VECTOR (31 downto 0); 
              wb_addr_i  : in  STD_LOGIC_VECTOR (3  downto 0); 
              wb_ack_o   : out STD_LOGIC; 
              wb_rty_o   : out STD_LOGIC; 
              wb_err_o   : out STD_LOGIC; 

              inst_o     : out STD_LOGIC_VECTOR (8*c_INST_LENGTH - 1 downto 0); 
              addr_o     : out STD_LOGIC_VECTOR (8*c_ADDR_LENGTH - 1 downto 0); 
              data_o     : out STD_LOGIC_VECTOR (8*c_DATA_LENGTH - 1 downto 0); 
              SPI0_o     : out STD_LOGIC_VECTOR (31 downto 0); 
              SPI1_o     : out STD_LOGIC_VECTOR (31 downto 0) 
           );

end spi_master_regs;

architecture Behavioral of spi_master_regs is

   signal s_SPI0      : STD_LOGIC_VECTOR (31  downto 0);
   signal s_SPI1      : STD_LOGIC_VECTOR (31  downto 0);

   signal s_wb_ack : STD_LOGIC;
   signal s_wb_rty : STD_LOGIC; 
   signal s_wb_err : STD_LOGIC; 

begin

   wb_ack_o <= s_wb_ack; 
   wb_rty_o <= s_wb_rty; 
   wb_err_o <= s_wb_err; 

   SPI0_o <= s_SPI0 ; 
   SPI1_o <= s_SPI1 ;

   p_wb: process (wb_clk_i)
   begin
      if rising_edge(wb_clk_i) then
         if wb_rst_i = '1' then
         elsif (wb_stb_i = '1' and wb_cyc_i = '1') then
            if (s_wb_ack or s_wb_rty or s_wb_err) = '1' then
               s_wb_ack   <= '0';
               s_wb_rty   <= '0';
               s_wb_err   <= '0';
            else
               case wb_we_i is
                  when '1' =>
                     case wb_addr_i is
                        when c_SPI0_addr =>
                           s_SPI0      <= wb_data_i;
                           s_wb_ack    <= '1'; 
                           s_wb_rty    <= '0'; 
                           s_wb_err    <= '0'; 
                        when c_SPI1_addr =>
                           s_SPI1      <= wb_data_i;
                           s_wb_ack    <= '1'; 
                           s_wb_rty    <= '0'; 
                           s_wb_err    <= '0'; 
                        when others      =>
                           s_wb_ack    <= '0'; 
                           s_wb_rty    <= '0'; 
                           s_wb_err    <= '1'; 
                     end case;
                  when others =>
                     case wb_addr_i is
                        when c_SPI0_addr =>
                           wb_data_o   <= s_SPI0;
                           s_wb_ack    <= '1'; 
                           s_wb_rty    <= '0'; 
                           s_wb_err    <= '0'; 
                        when c_SPI1_addr =>
                           wb_data_o   <= s_SPI1;
                           s_wb_ack    <= '1'; 
                           s_wb_rty    <= '0'; 
                           s_wb_err    <= '0'; 
                        when others      =>
                     end case;
               end case;
            end if;
         else
         end if;
      else
      end if;
   end process;

end Behavioral;
