----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    17:10:27 06/15/2012 
-- Design Name:     
-- Module Name:    spi_master_top - Behavioral 
-- Project Name:   CTDAH/SVEC
-- Target Devices: Spartan 6
-- Tool versions:  Xilinx ISE 13.4
-- Description:    This is a spi_master with programable field length and progra
--                 mable setup and hold chip select times.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use work.spi_master_pkg.ALL;


entity spi_master_top is
   port(
          wb_clk       : in   STD_LOGIC;
          wb_rst_i     : in   STD_LOGIC;

          wb_stb_i     : in   STD_LOGIC;
          wb_cyc_i     : in   STD_LOGIC;
          wb_sel_i     : in   STD_LOGIC_VECTOR(3  downto 0);
          wb_we_i      : in   STD_LOGIC;
          wb_data_i    : in   STD_LOGIC_VECTOR(31 downto 0);
          wb_data_o    : out  STD_LOGIC_VECTOR(31 downto 0);
          wb_addr_i    : in   STD_LOGIC_VECTOR(3  downto 0);
          wb_ack_o     : out  STD_LOGIC;
          wb_rty_o     : out  STD_LOGIC;
          wb_err_o     : out  STD_LOGIC;

          spi_mosi_o   : out  STD_LOGIC;
          spi_miso_i   : in   STD_LOGIC;
          spi_clk_o    : out  STD_LOGIC;
          spi_cs_n_o   : out  STD_LOGIC

       );
end spi_master_top;

architecture Behavioral of spi_master_top is

   component spi_master_core is
   generic(
             g_INST_LENGTH  : NATURAL := c_INST_LENGTH;
             g_ADDR_LENGTH  : NATURAL := c_ADDR_LENGTH;
             g_DATA_LENGTH  : NATURAL := c_DATA_LENGTH;
             g_CLK_I_PERIOD : NATURAL := c_CLK_I_PERIOD);
   port(
          rst_i         : in  STD_LOGIC;
          clk_i         : in  STD_LOGIC;

          inst_i        : in  STD_LOGIC_VECTOR (8*g_INST_LENGTH - 1 downto 0);
          addr_i        : in  STD_LOGIC_VECTOR (8*g_ADDR_LENGTH - 1 downto 0);
          data_i        : in  STD_LOGIC_VECTOR (8*g_DATA_LENGTH - 1 downto 0);


          SPI0_i        : in  STD_LOGIC_VECTOR (31 downto 0);
          SPI1_i        : in  STD_LOGIC_VECTOR (31 downto 0);

          spi_mosi_o    : out STD_LOGIC;
          spi_miso_i    : in  STD_LOGIC;
          spi_clk_o     : out STD_LOGIC;
          spi_cs_n_o    : out STD_LOGIC
       );
   end component;

   component spi_master_regs is
      port(
      wb_rst_i   : in  STD_LOGIC;
      wb_clk_i   : in  STD_LOGIC;
                       
      wb_stb_i   : in  STD_LOGIC;
      wb_cyc_i   : in  STD_LOGIC;
      wb_sel_i   : in  STD_LOGIC_VECTOR (3  downto 0);
      wb_we_i    : in  STD_LOGIC;
      wb_data_i  : in  STD_LOGIC_VECTOR (31 downto 0);
      wb_data_o  : out STD_LOGIC_VECTOR (31 downto 0);
      wb_addr_i  : in  STD_LOGIC_VECTOR (3  downto 0);
      wb_ack_o   : out STD_LOGIC;
      wb_rty_o   : out STD_LOGIC;
      wb_err_o   : out STD_LOGIC;
                       
      inst_o     : out STD_LOGIC_VECTOR (8*c_INST_LENGTH - 1 downto 0);
      addr_o     : out STD_LOGIC_VECTOR (8*c_ADDR_LENGTH - 1 downto 0);
      data_o     : out STD_LOGIC_VECTOR (8*c_DATA_LENGTH - 1 downto 0);
      SPI0_o     : out STD_LOGIC_VECTOR (31 downto 0);
      SPI1_o     : out STD_LOGIC_VECTOR (31 downto 0)
   );

   end component;


signal s_inst : STD_LOGIC_VECTOR (8*c_INST_LENGTH - 1 downto 0); 
signal s_addr : STD_LOGIC_VECTOR (8*c_ADDR_LENGTH - 1 downto 0); 
signal s_data : STD_LOGIC_VECTOR (8*c_DATA_LENGTH - 1 downto 0); 
signal s_SPI0 : STD_LOGIC_VECTOR (31  downto 0); 
signal s_SPI1 : STD_LOGIC_VECTOR (31  downto 0); 

begin

   inst_spi_master_core: spi_master_core 
   port map(
              rst_i       => wb_rst_i, 
              clk_i       => wb_clk, 

              inst_i      => s_inst, 
              addr_i      => s_addr, 
              data_i      => s_data, 
              SPI0_i      => s_SPI0, 
              SPI1_i      => s_SPI1, 

              spi_mosi_o  => spi_mosi_o, 
              spi_miso_i  => spi_miso_i, 
              spi_clk_o   => spi_clk_o, 
              spi_cs_n_o  => spi_cs_n_o 
              );

   inst_spi_master_regs: spi_master_regs
   port map(
              wb_rst_i    => wb_rst_i, 
              wb_clk_i    => wb_clk,

              wb_stb_i    => wb_stb_i, 
              wb_cyc_i    => wb_cyc_i, 
              wb_sel_i    => wb_sel_i, 
              wb_we_i     => wb_we_i, 
              wb_data_i   => wb_data_i, 
              wb_data_o   => wb_data_o, 
              wb_addr_i   => wb_addr_i, 
              wb_ack_o    => wb_ack_o, 
              wb_rty_o    => wb_rty_o, 
              wb_err_o    => wb_err_o, 

              inst_o      => s_inst, 
              addr_o      => s_addr, 
              data_o      => s_data, 
              SPI0_o      => s_SPI0, 
              SPI1_o      => s_SPI1
           );

end Behavioral;

